/* Copyright 2022 Sensaru GmbH */

#ifndef LIBC1_UTIL_SRC_DATETIME_H_
#define LIBC1_UTIL_SRC_DATETIME_H_

#include <ctime>
#include <chrono>
#include <string>
#include <unordered_map>
#include <array>

namespace C1Util {

class DateTime {
 public:
  DateTime() = delete;

  template<typename Duration = std::chrono::milliseconds>
  static int64_t GetUtc();

  /**
   *
   * @tparam Duration The duration to use. Supported are `std::chrono::seconds`, `std::chrono::milliseconds`,
   *                  `std::chrono::microseconds` and `std::chrono::nanoseconds`.
   * @param utc_time The optional UTC time in `Duration` since January 1, 1970. Pass "0" to get the current time.
   * @param linear Returns "linear" local time. I. e. when DST ends, the repeated two hours are squeezed into one hour.
   *               As an example the first hour from 2 to 3 am becomes 2 to 2:30 am, the second hour from 2 to 3 am
   *               becomes 2:30 to 3 am.
   * @param timezone The optional timezone string (e. g. "Europe/Berlin")
   * @return Returns the time in `Duration` since January 1, 1970.
   */
  template<typename Duration = std::chrono::milliseconds>
  static int64_t GetLocal(int64_t utc_time = 0, bool linear = false, const std::string &timezone = "");

  template<typename Duration = std::chrono::milliseconds>
  static std::tm GetDate(int64_t time = 0);

  static bool IsStartOfYear(const std::tm &date);

  /**
   * Returns a formatted time string.
   *
   * @tparam Duration The duration to use. Supported are `std::chrono::seconds`, `std::chrono::milliseconds`,
   *                  `std::chrono::microseconds` and `std::chrono::nanoseconds`.
   * @param format See https://en.cppreference.com/w/c/chrono/strftime for a list of format options.
   *               Example: `"%Y-%m-%d %H:%M:%S"`
   * @param time The optional time in `Duration` since January 1, 1970.
   * @return Returns the formatted time string.
   */
  template<typename Duration = std::chrono::milliseconds>
  static std::string GetString(const std::string &format, int64_t time = 0);

  static std::string MccToTimezone(uint16_t mcc);
};

}

#endif //LIBC1_UTIL_SRC_TIME_HPP_
